SystemOrganization addCategory: #'AL-Http-Core'!
SystemOrganization addCategory: #'AL-Http-Endpoints'!
SystemOrganization addCategory: #'AL-Http-Protocol'!

Error subclass: #ALMalformedRequest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

Object subclass: #ALClient
	instanceVariableNames: 'host port in out handler'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/6/2011 16:38'!
host: aString
	^ self host: aString port: 80 ! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:09'!
host: aString port: anInteger
	^ self host: aString port: anInteger in: nil out: nil! !

!ALClient class methodsFor: 'as yet unclassified' stamp: 'cwp 9/4/2011 00:08'!
host: aString port: anInteger in: aReadStream out: aWriteStream
	^ self basicNew
		initializeWithHost: aString
		port: anInteger
		in: aReadStream
		out: aWriteStream! !

!ALClient methodsFor: 'private' stamp: 'cwp 9/6/2011 17:11'!
addHostToRequest: aRequest
	| ws |
	ws := String new writing.
	ws write: host.
	port = 80 ifFalse:
		[ws put: $:.
		ws print: port].
	aRequest host: ws conclusion! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:13'!
connect
	self isConnected ifFalse:
		[| sock |
		Socket initializeNetwork.
		sock := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
		sock connectToHostNamed: host port: port.
		in := sock reading.
		out := sock writing]! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 16:32'!
disconnect
	in ifNotNil:
		[in close.
		in := nil].
	out ifNotNil:
		[out close.
		out := nil]! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/6/2011 15:50'!
get: aPath
	| request response |
	request := ALRequest get: aPath.
	response := self sendRequest: request.
	^ response content! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/8/2011 15:40'!
handleRequest: aRequest
	| parser response printer |
	self connect.
	printer := ALHttpPrinter on: out.
	parser := ALHttpParser on: in.
	printer printRequest: aRequest.
	response := parser parseResponse.
	response body body: [in reading].
	^ response! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
host
	^ host! !

!ALClient methodsFor: 'initialize-release' stamp: 'cwp 9/11/2011 15:10'!
initializeWithHost: aString port: anInteger in: aReadStream out: aWriteStream
	self initialize.
	host := aString.
	port := anInteger.
	in := aReadStream.
	out := aWriteStream.
	handler := ALChunkingRelay destination: self.
	handler := ALLimitingRelay destination: handler.
	handler := ALResponseDecodingRelay destination: handler.
	handler := ALHostRelay destination: handler host: host port: port.
! !

!ALClient methodsFor: 'connection' stamp: 'cwp 9/6/2011 17:03'!
isConnected
	in ifNil: [^ false].
	out ifNil: [^ false].
	^ true! !

!ALClient methodsFor: 'accessing' stamp: 'cwp 9/4/2011 00:10'!
port
	^ port! !

!ALClient methodsFor: 'public' stamp: 'cwp 9/7/2011 13:16'!
sendRequest: aRequest
	^ handler handleRequest: aRequest! !

Object subclass: #ALConnection
	instanceVariableNames: 'in out application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALConnection class methodsFor: 'as yet unclassified' stamp: 'cwp 4/24/2011 23:17'!
in: aReadStream out: aWriteStream application: anApplication
	^ self basicNew 
		initializeWithIn: aReadStream 
		out: aWriteStream 
		application: anApplication ! !

!ALConnection class methodsFor: 'instance creation' stamp: 'cwp 4/24/2011 23:17'!
socket: aSocket application: anApplication
	^ self in: aSocket reading out: aSocket writing application: anApplication! !

!ALConnection methodsFor: 'as yet unclassified' stamp: 'cwp 4/13/2011 21:54'!
close
	in close.
	out close.
	! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/20/2011 23:59'!
handleRequest
	| parser printer response |
	parser := ALHttpParser on: in.
	printer := ALHttpPrinter on: out.
	response := self handleRequest: parser parseRequest.
	printer printResponse: response.
	response writeEntityInto: out.
	response isConnectionClose ifTrue: [ self close ]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 8/21/2011 00:01'!
handleRequest: aRequest
	^ [application handleRequest: aRequest]
		on: Error
		do: [:error | ALResponse status: 500 reason: error messageText]! !

!ALConnection methodsFor: 'running' stamp: 'cwp 4/13/2011 21:57'!
handleRequests
	[
		[[self handleRequest] repeat]
			on: Incomplete
			do: [:ex | self close]	
	] ifCurtailed: [self close]! !

!ALConnection methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 23:16'!
initializeWithIn: aReadStream out: aWriteStream application: anApplication
	self initialize.
	in := aReadStream.
	out := aWriteStream.
	application := anApplication ! !

Object subclass: #ALContentType
	instanceVariableNames: 'type subtype parameters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALContentType class methodsFor: 'instance creation' stamp: 'cwp 9/8/2011 14:28'!
html
	^ (self type: 'text' subtype: 'html')
		charset: 'utf-8';
		yourself! !

!ALContentType class methodsFor: 'instance creation' stamp: 'cwp 9/11/2011 12:04'!
plainAscii
	^ (self type: 'text' subtype: 'plain')
		charset: 'ascii';
		yourself! !

!ALContentType class methodsFor: 'instance creation' stamp: 'cwp 9/7/2011 15:56'!
type: typeString subtype: subtypeString
	^ self basicNew
		initializeWithType: typeString subtype: subtypeString! !

!ALContentType methodsFor: 'converting' stamp: 'cwp 9/8/2011 14:31'!
asByteArray
	| ws |
	ws := ByteArray new writing.
	ws
		write: type;
		put: $/ asciiValue;
		write: subtype.
	parameters keysAndValuesDo:
		[:attribute :value |
		ws
			write: #[59 32];
			write: attribute;
			put: 61;
			write: value].
	^ ws conclusion! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/8/2011 14:15'!
charset
	^ (self parameter: 'charset' ifAbsent: [^ nil]) asLowercase asSymbol! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/8/2011 14:29'!
charset: aString
	self parameter: 'charset' value: aString asSymbol! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/8/2011 14:18'!
hasCharset
	^ parameters includesKey: 'charset' ! !

!ALContentType methodsFor: 'initialize-release' stamp: 'cwp 9/7/2011 15:59'!
initializeWithType: typeString subtype: subtypeString
	self initialize.
	type := typeString asLowercase.
	subtype := subtypeString asLowercase.
	parameters := Dictionary new.! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:01'!
isHtml
	^ self isText and: [subtype = 'html']! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:09'!
isImage
	^ type = 'image'! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:10'!
isJpeg
	^ self isImage and: [subtype = 'jpeg']! !

!ALContentType methodsFor: 'testing' stamp: 'cwp 9/7/2011 16:01'!
isText
	^ self type = 'text'! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:15'!
parameter: aString
	^ self parameter: aString ifAbsent: [nil]! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:15'!
parameter: aString ifAbsent: aBlock
	^ parameters at: aString ifAbsent: aBlock! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:13'!
parameter: pString value: vString
	parameters at: pString put: vString! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:00'!
subtype
	^ subtype ! !

!ALContentType methodsFor: 'accessing' stamp: 'cwp 9/7/2011 16:00'!
type
	^ type! !

Object subclass: #ALHttpParser
	instanceVariableNames: 'in ch message'
	classVariableNames: 'CarriageReturn Colon Digits Equals LineFeed Semicolon Slash Space Token Version'
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 9/8/2011 10:49'!
initialize
	"Initialize character classes with their ASCII values"

	CarriageReturn := 13.
	Colon := 58.
	Equals := 61.
	LineFeed := 10.
	Semicolon := 59.
	Slash := 47.
	Space := 32.
	Digits := #[48 49 50 51 52 53 54 55 56 57].	
	Version := #[46], Digits.
	Token := #[33 35 36 37 38 39 42 43 45 46 48 49 50 51 52 53 54 55 56 57 
				65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 
				85 86 87 88 89 90 94 95 96 97 98 99 100 101 102 103 104 105 
				106 107 108 109 110 111 112 113 114 115 116 117 118 119 120
				121 122 124 126].
	
	
! !

!ALHttpParser class methodsFor: 'as yet unclassified' stamp: 'cwp 4/5/2011 19:51'!
on: aReadStream
	^ self basicNew initializeWithStream: aReadStream! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assert: aByte
	ch = aByte ifFalse: [ALMalformedRequest signal].
	ch := nil.! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertCrLf
	"Character cr asciiValue = 13"
	"Character lf acsiiValue = 10"
	
	self assert: 13.
	self next: 10.
! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 4/5/2011 21:08'!
assertSpace
	"Character space asciiValue = 32"
	self assert: 32.
! !

!ALHttpParser methodsFor: 'initialize-release' stamp: 'cwp 4/5/2011 19:52'!
initializeWithStream: aReadStream
	in := aReadStream
	! !

!ALHttpParser methodsFor: 'asserting' stamp: 'cwp 9/4/2011 23:42'!
next: expected
	| actual |
	actual := in get.
	actual = expected ifFalse: [ALMalformedRequest signal]! !

!ALHttpParser methodsFor: 'entry points' stamp: 'cwp 9/7/2011 16:26'!
parseContentType
	message := self parseTypeSubtype.
	self parseParameters.
	^ message! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:53'!
parseFieldValue
	"Character tab asciiValue = 9"
	"Character space asciiValue = 32"
	"Character cr asciivalue = 13"
	"Character lf asciiValue = 10"
	"For now, we ignore the possibility of line folding"

	| out byte |
	[#(32 9) includes: (byte := in get)] whileTrue.
	out := ByteArray new writing.
	out put: byte.
	[(byte := in get) = CarriageReturn] whileFalse: [out put: byte].
	self next: LineFeed.
	^ out conclusion! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 14:01'!
parseHeaders
	
	| name value |
	[name := self parseToken.
	name isEmpty ifTrue: [^ self assertCrLf].
	self assert: Colon.
	value := self parseFieldValue.
	message rawHeader: name put: value]
		repeat
	
! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 19:58'!
parseMethod
	| method |
	method := self parseToken.
	self assertSpace.
	^ method! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:53'!
parseParameter
 
	| attribute value |
	in available = 0 ifTrue: [^ false].
	self assert: Semicolon.
	self next: Space.
	attribute := self parseToken.
	self assert: Equals.
	value := self parseToken.
	message parameter: attribute value: value.
	^ true! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:30'!
parseParameters
	
	[[self parseParameter] whileTrue]
		on: Incomplete
		do: [:error]! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:43'!
parsePath

	| out path |
	path := ALPath root.
	ch := in get. 
	ch = Slash ifFalse: [self error].

	out := String new writing.
	[(ch := in get) = Space] whileFalse:
		[ch = Slash
			ifFalse: [out put: (Character value: ch)]
			ifTrue: 
				[path := path / out conclusion.
				out := String new writing]].
	path := path / out conclusion.
	self assertSpace.
	^ path
			! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/4/2011 23:48'!
parseReason
	^ (in ending: 13) rest asString! !

!ALHttpParser methodsFor: 'entry points' stamp: 'cwp 9/4/2011 23:50'!
parseRequest
	message := self parseRequestLine.
	self parseHeaders.
	^ message! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/7/2011 14:48'!
parseRequestLine
	| version path method entity |
	method := self parseMethod.
	path := self parsePath.
	version := self parseVersion.
	self assertCrLf.
	entity := ALEntity empty.
	^ ALRequest method: method path: path version: version body: entity.
	! !

!ALHttpParser methodsFor: 'entry points' stamp: 'cwp 9/4/2011 23:51'!
parseResponse
	message := self parseStatusLine.
	self parseHeaders.
	^ message! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:36'!
parseStatus
	| digits |
	digits := self readAll: Digits.
	^ Integer readFrom: digits readStream! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:45'!
parseStatusLine
	| version reason status body |
	version := self parseVersion.
	self assert: Space.
	status := self parseStatus.
	self assert: Space.
	reason := self parseReason.
	self next: LineFeed.
	body := ALEntity empty.
	^ ALResponse status: status reason: reason version: version body: body! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:45'!
parseToken

	|  out |
	out := ByteArray new writing.
	ch ifNotNil: [(Token includes: ch) ifTrue: [out put: ch]].
	[[Token includes: (ch := in get)] whileTrue: [out put: ch]]
		on: Incomplete
		do: [:error].
	^ out conclusion asString! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:45'!
parseTypeSubtype

	| type subtype |
	type := self parseToken.
	self assert: Slash.
	subtype := self parseToken.
	^ ALContentType type: type subtype: subtype! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 9/8/2011 10:46'!
parseVersion
	| version |
	self next: $H asciiValue.
	self next: $T asciiValue.
	self next: $T asciiValue.
	self next: $P asciiValue.
	self next: Slash.
	version := self readAll: Version.
	^ version! !

!ALHttpParser methodsFor: 'parsing' stamp: 'cwp 4/5/2011 20:12'!
readAll: aByteArray
	| out |
	out := ByteArray new writing.
	[aByteArray includes: (ch := in get)] whileTrue: [out put: ch].
	^ out conclusion asString! !

Object subclass: #ALHttpPrinter
	instanceVariableNames: 'out message'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Protocol'!

!ALHttpPrinter class methodsFor: 'as yet unclassified' stamp: 'cwp 4/23/2011 15:54'!
on: aWriteStream
	^ self basicNew initializeWithStream: aWriteStream ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:20'!
crlf

	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'initialize-release' stamp: 'cwp 4/23/2011 15:55'!
initializeWithStream: aWriteStream
	out := aWriteStream  ! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:21'!
printHeader: name value: value
	out
		write: name;
		put: 58;
		put: 32;
		write: value;
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/8/2011 11:11'!
printHeaders
	message rawHeadersDo: [ :name :value | self printHeader: name value: value ].
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:39'!
printRequest
	self printRequestLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:30'!
printRequest: aRequest 
	message := aRequest.
	self printRequest.
	message := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 23:21'!
printRequestLine
	"Character space asciiValue -> 32"

	out
		write: message method;
		space;
		print: message path;
		space.
	self printVersion.
	out
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/6/2011 16:26'!
printResponse
	self printStatusLine.
	self printHeaders.
	out flush! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:30'!
printResponse: aResponse
	message := aResponse.
	self printResponse.
	message := nil! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:35'!
printStatusLine
	self printVersion.
	out
		space;
		print: message status;
		space;
		write: message reason;
		cr;
		lf! !

!ALHttpPrinter methodsFor: 'printing' stamp: 'cwp 9/4/2011 01:35'!
printVersion
	out
		write: 'HTTP/';
		write: (String streamContents: [ :s | message version printFractionAsDecimalOn: s ])! !

Object subclass: #ALMimeMessage
	instanceVariableNames: 'headers body'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

ALMimeMessage subclass: #ALEntity
	instanceVariableNames: 'filters'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALEntity class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 16:18'!
body: anObject
	^ self basicNew initializeWithBody: anObject! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/8/2011 14:46'!
callback: aBlock
	^ self body: aBlock! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 9/8/2011 14:46'!
empty
	^ ALEntity body: nil! !

!ALEntity class methodsFor: 'writing' stamp: 'cwp 4/23/2011 19:47'!
html: aString
	^ (self callback: [:out | (out encoding: #ascii) write: aString])
		header: 'Content-Type' put: 'text/html';
		header: 'Content-Length' put: aString length printString;
		yourself! !

!ALEntity class methodsFor: 'reading' stamp: 'cwp 9/8/2011 14:47'!
reading: aReadable
	^ self body: [aReadable reading]! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/6/2011 15:57'!
contentEncoding
	^ self header: 'Content-Encoding'! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 23:47'!
contentLength
	| bytes |
	bytes := (self header: 'Content-Length') ifNil: [^ nil].
	^ Integer readFrom: bytes asString readStream! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 23:47'!
contentLength: anInteger 
	self header: 'Content-Length' put: anInteger printString asByteArray! !

!ALEntity methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 14:37'!
contentType
	| bytes |
	bytes := (self rawHeader: 'Content-Type') ifNil: [^ nil].
	^ (ALHttpParser on: bytes reading) parseContentType ! !

!ALEntity methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 14:33'!
contentType: aContentType 
	self rawHeader: 'Content-Type' put: aContentType asByteArray! !

!ALEntity methodsFor: 'accessing' stamp: 'cwp 9/7/2011 15:16'!
filter
	| catcher |
	catcher := MessageCatcher new.
	catcher privAccumulator: filters.
	^ catcher
		! !

!ALEntity methodsFor: 'initialize-release' stamp: 'cwp 9/7/2011 15:06'!
initializeWithBody: anObject
	super initializeWithBody: anObject.
	filters := OrderedCollection new    ! !

!ALEntity methodsFor: 'io' stamp: 'cwp 9/12/2011 17:25'!
readWith: aBlock
	| stream |
	body ifNil: [^ nil].
	stream := filters
		inject: body value
		into: [:s :message | message sendTo: s].
	^ [aBlock value: stream] ensure: 
		[(filters anySatisfy: [:m | m selector = #limiting:]) ifTrue: [stream rest]]  ! !

!ALEntity methodsFor: 'io' stamp: 'cwp 9/11/2011 14:55'!
writeInto: aWriteStream
	| stream |
	body ifNil: [^ nil].
	stream := filters reversed
		inject: aWriteStream
		into: [:s :message | message sendTo: s].
	^ [body value: stream]
		ensure: [stream flush].
! !

ALMimeMessage subclass: #ALHttpMessage
	instanceVariableNames: 'version'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALHttpMessage class methodsFor: 'constants' stamp: 'cwp 9/5/2011 14:20'!
defaultVersion
	^ 1.1s1! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 10:58'!
beConnectionKeepAlive
	self header: 'Connection' put: #[107 101 101 112 45 97 108 105 118 101]! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 5/4/2011 22:02'!
beTransferEncodingChunked
	self header: 'Transfer-Encoding' put: 'chunked'! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:50'!
content
	^ self readEntityWith: [:in | in rest]! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 23:46'!
contentLength
	^ body contentLength! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 23:46'!
contentLength: anInteger
	^ body contentLength: anInteger! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 9/8/2011 14:29'!
contentType
	^ body contentType! !

!ALHttpMessage methodsFor: 'headers - setting' stamp: 'cwp 9/8/2011 14:29'!
contentType: anObject
	^ body contentType: anObject! !

!ALHttpMessage methodsFor: 'headers - getting' stamp: 'cwp 9/9/2011 00:34'!
host
	^ self header: 'Host'! !

!ALHttpMessage methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:11'!
initializeWithVersion: aScaledDecimal body: anEntity
	self initializeWithBody: anEntity.
	version := aScaledDecimal ! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 9/8/2011 14:57'!
isConnectionClose
	^ (self rawHeader: 'Connection' ifAbsent: [^ false]) = #[99 108 111 115 101]! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 9/8/2011 15:01'!
isConnectionKeepAlive
	"'keep-alive' asByteArray"
	
	^ (self rawHeader: 'Connection' ifAbsent: [^ true]) 
		= #[107 101 101 112 45 97 108 105 118 101]! !

!ALHttpMessage methodsFor: 'headers - testing' stamp: 'cwp 5/2/2011 18:37'!
isTransferEncodingChunked
	^ (self header: 'Transfer-Encoding') = 'chunked'! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:56'!
rawHeader: aString ifAbsent: aBlock
	^ (self class entityHeaders includes: aString)
		ifTrue: [body rawHeader: aString ifAbsent: aBlock]
		ifFalse: [headers at: aString ifAbsent: aBlock]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:21'!
rawHeader: aString put: aByteArray
	(self class entityHeaders includes: aString)
		ifTrue: [body rawHeader: aString put: aByteArray]
		ifFalse: [headers at: aString put: aByteArray]! !

!ALHttpMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:11'!
rawHeadersDo: aBlock
	super rawHeadersDo: aBlock.
	body rawHeadersDo: aBlock.! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/8/2011 23:50'!
readEntityFrom: aReadStream
	body body: [aReadStream]! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/6/2011 15:51'!
readEntityWith: aBlock
	^ body readWith: aBlock! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version
	^ version! !

!ALHttpMessage methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:16'!
version: anObject
	version := anObject! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/8/2011 14:52'!
writeEntityInto: aWriteStream
	body writeInto: aWriteStream! !

!ALHttpMessage methodsFor: 'entity' stamp: 'cwp 9/8/2011 23:48'!
writeEntityWith: aBlock
	body body: aBlock! !

ALHttpMessage subclass: #ALRequest
	instanceVariableNames: 'method path'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALRequest class methodsFor: 'convenience' stamp: 'cwp 9/4/2011 01:58'!
get: aPath 
	^ (self method: 'GET' path: aPath)
		userAgent: self userAgent;
		acceptAnything;
		yourself! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:21'!
method: aString path: aPath
	^ self
		method: aString
		path: aPath
		version: self defaultVersion
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:20'!
method: aString path: aPath version: aScaledDecimal 
	^ self 
		method: aString 
		path: aPath 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALRequest class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:19'!
method: aString path: aPath version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithMethod: aString 
		path: aPath 
		version: aScaledDecimal 
		body: anEntity ! !

!ALRequest class methodsFor: 'accessing' stamp: 'cwp 9/4/2011 02:01'!
userAgent
	^ 'Altitude/0.1'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:02'!
accept: aString
	self header: 'Accept' put: aString! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:00'!
acceptAnything
	self accept: '*/*'! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/6/2011 17:09'!
host: aString
	self header: 'Host' put: aString! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:05'!
initializeWithMethod: aString path: aPath 
	self initializeWithBody: ALEntity empty.
	method := aString.
	path := aPath.
	! !

!ALRequest methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:13'!
initializeWithMethod: aString path: aPath version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	method := aString.
	path := aPath.
	! !

!ALRequest methodsFor: 'testing' stamp: 'cwp 3/21/2011 22:25'!
isMethodGet
	^ method = 'GET'! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 4/24/2011 21:01'!
method
	^ method! !

!ALRequest methodsFor: 'accessing' stamp: 'cwp 3/20/2011 22:58'!
path
	^ path! !

!ALRequest methodsFor: 'headers - setting' stamp: 'cwp 9/4/2011 02:02'!
userAgent: aString
	self header: 'User-Agent' put: aString! !

ALHttpMessage subclass: #ALResponse
	instanceVariableNames: 'status reason'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Core'!

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/13/2011 23:01'!
notFound: aPath
	^ self status: 404 reason: 'Not found ', aPath printString! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 9/9/2011 00:06'!
ok
	^ self status: 200 reason: 'OK'! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 8/9/2011 16:00'!
seeOther: aPath
	^ (self status: 303 reason: 'See Other')
		header: 'Location' put: aPath printString;
		yourself! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:17'!
status: anInteger reason: aString
	^ self 	
		status: anInteger 
		reason: aString 
		version: self defaultVersion 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString body: anEntity
	^ self 
		status: anInteger 
		reason: aString 
		version: self defaultVersion  
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 4/23/2011 19:43'!
status: status reason: aString details: aString2
	| html |
	html := String new writing
		write: '<!!DOCTYPE html><html><head><title>';
		print: status;
		space;
		write: aString;
		write: '</title></head><body><h1>';
		write: aString;
		write: '</h1><p>';
		write: aString2;
		write: '</p></body></html>'.
	^ self status: 404 reason: 'Not Found' body: (ALEntity html: html)! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:18'!
status: anInteger reason: aString version: aScaledDecimal
	^ self 
		status: anInteger 
		reason: aString 
		version: aScaledDecimal 
		body: ALEntity empty! !

!ALResponse class methodsFor: 'instance creation' stamp: 'cwp 9/5/2011 14:14'!
status: anInteger reason: aString version: aScaledDecimal body: anEntity
	^ self basicNew
		initializeWithStatus: anInteger 
		reason: aString
		version: aScaledDecimal
		body: anEntity! !

!ALResponse class methodsFor: 'common responses' stamp: 'cwp 5/2/2011 18:27'!
type: aString callback: aBlock
	| entity |
	entity := ALEntity callback: aBlock.
	entity contentType: aString.
	^ (self status: 200 reason: 'OK' body: entity)
		beTransferEncodingChunked;
		yourself! !

!ALResponse methodsFor: 'initialize-release' stamp: 'cwp 9/5/2011 14:29'!
initializeWithStatus: anInteger reason: aString version: aScaledDecimal body: anEntity
	self initializeWithVersion: aScaledDecimal body: anEntity.
	status := anInteger.
	reason := aString.
	! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
reason
	^ reason! !

!ALResponse methodsFor: 'accessing' stamp: 'cwp 4/23/2011 16:23'!
status
	^ status! !

!ALMimeMessage class methodsFor: 'as yet unclassified' stamp: 'cwp 9/5/2011 14:45'!
entityHeaders
	^ #(
		'Content-Encoding' 
		'Content-Language'
		'Content-Length'
		'Content-Location'
		'Content-MD5'
		'Content-Range'
		'Content-Type'
		'Expires'
		'Last-Modified'
	)! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 9/7/2011 14:47'!
body
	^ body! !

!ALMimeMessage methodsFor: 'accessing' stamp: 'cwp 5/2/2011 18:33'!
body: anObject
	 body := anObject! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:06'!
header: aString
	^ (self rawHeader: aString) ifNotNil: [:value | value asString]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:09'!
header: aString put: anObject
	self rawHeader: aString put: anObject asByteArray! !

!ALMimeMessage methodsFor: 'initialize-release' stamp: 'cwp 4/24/2011 21:04'!
initializeWithBody: anObject
	self initialize.
	headers := Dictionary new.
	body := anObject.! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:56'!
rawHeader: aString
	^ self rawHeader: aString ifAbsent: [ nil ]! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 14:54'!
rawHeader: aString ifAbsent: aBlock
	^ headers at: aString ifAbsent: aBlock! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:08'!
rawHeader: aString put: aByteArray
	headers at: aString put: aByteArray! !

!ALMimeMessage methodsFor: 'headers - accessing' stamp: 'cwp 9/8/2011 11:11'!
rawHeadersDo: aBlock
	headers keysAndValuesDo: aBlock  ! !

Object subclass: #ALRelay
	instanceVariableNames: 'destination'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

ALRelay subclass: #ALChunkingRelay
	instanceVariableNames: 'size'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALChunkingRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/12/2011 17:23'!
destination: aMessageHandler
	^ self destination: aMessageHandler size: 4096   ! !

!ALChunkingRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:42'!
destination: aMessageHandler size: anInteger
	^ self basicNew
		initializeWithDestination: aMessageHandler 
		size: anInteger  ! !

!ALChunkingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:44'!
annotate: aMessage
	aMessage isTransferEncodingChunked ifTrue:
		[aMessage body filter chunking: size]! !

!ALChunkingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:42'!
initializeWithDestination: aMessageHandler size: anInteger
	self initializeWithDestination: aMessageHandler.
	size := anInteger  ! !

ALRelay subclass: #ALDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 12:19'!
annotate: aMessage
	| body |
	body := aMessage body.
	body contentType ifNotNil: 
		[ :type | 
		type hasCharset ifTrue: 
			[ body filter encoding: type charset ] ].
	
	 ^ aMessage! !

ALDecodingRelay subclass: #ALRequestDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALRequestDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 11:42'!
handleRequest: aRequest
	^ destination handleRequest: (self annotate: aRequest)! !

ALDecodingRelay subclass: #ALResponseDecodingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALResponseDecodingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 11:43'!
handleRequest: aRequest
	^ self annotate: (destination handleRequest: aRequest)! !

ALRelay subclass: #ALHostRelay
	instanceVariableNames: 'value'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALHostRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 13:14'!
destination: aMessageHandler host: aString port: anInteger
	^ self basicNew 
		initializeWithDestination: aMessageHandler
		host: aString
		port: anInteger! !

!ALHostRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 13:16'!
handleRequest: aRequest
	aRequest host: value.
	^ destination handleRequest: aRequest! !

!ALHostRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 13:15'!
initializeWithDestination: aMessageHandler host: host port: port
	| ws |
	self initializeWithDestination: aMessageHandler.
	ws := String new writing.
	ws write: host.
	port = 80 ifFalse:
		[ws put: $:.
		ws print: port].
	value := ws conclusion  ! !

ALRelay subclass: #ALLimitingRelay
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALLimitingRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:52'!
annotate: aMessage
	aMessage contentLength ifNotNil: 
		[ :length | aMessage body filter limiting: length ]! !

!ALRelay class methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 15:25'!
destination: aMessageHandler
	^ self basicNew initializeWithDestination: aMessageHandler ! !

!ALRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:50'!
annotate: aMessage! !

!ALRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/11/2011 14:50'!
handleRequest: aRequest
	| response |
	self annotate: aRequest.
	response := destination handleRequest: aRequest.
	self annotate: response.
	^ response ! !

!ALRelay methodsFor: 'as yet unclassified' stamp: 'cwp 9/7/2011 13:13'!
initializeWithDestination: aMessageHandler
	self initialize.
	destination := aMessageHandler! !

Object subclass: #ALServer
	instanceVariableNames: 'port process socket state application'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'AL-Http-Endpoints'!

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:13'!
initialize
	Smalltalk addToStartUpList: self after: AutoStart.
	Smalltalk addToShutDownList: self! !

!ALServer class methodsFor: 'instance creation' stamp: 'cwp 4/23/2011 17:55'!
on: anInteger application: anApplication
	^ self basicNew initializeWithPort: anInteger application: anApplication! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
shutDown: aBoolean
	self allInstancesDo: [:ea | ea shutDown]! !

!ALServer class methodsFor: 'system startup' stamp: 'cwp 5/4/2011 20:57'!
startUp: aBoolean
	self allInstancesDo: [:ea | ea startUp]! !

!ALServer methodsFor: 'initialize-release' stamp: 'cwp 5/4/2011 20:26'!
initializeWithPort: anInteger application: anApplication
	port := anInteger.
	application := anApplication.
	state := #stopped
	! !

!ALServer methodsFor: 'testing' stamp: 'cwp 4/13/2011 17:01'!
isRunning
	^ state == #running! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:23'!
loop
	[self serveConnection] repeat! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:16'!
printOn: aStream
	aStream nextPutAll: (self streamingPrintOn: String new writing)! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 20:27'!
run

	socket := Socket tcpCreateIfFail: [self error: 'Unable to create a socket'].
	socket
		setOption: 'TCP_NODELAY' value: 1;
		setOption: 'SO_SNDBUF' value: 1024;
		setOption: 'SO_RCVBUF' value: 1024;
		listenOn: port backlogSize: 10.
	socket isValid ifFalse: [self error: 'Unable to listen on port ', port printString].
	
	[self loop] ifCurtailed:   
		[socket destroy.
		socket := nil.
		process := nil].

! !

!ALServer methodsFor: 'private' stamp: 'cwp 5/4/2011 21:42'!
serveConnection
	| connection |
	connection := socket waitForAcceptFor: self timeout.
	connection ifNil: [^ self].
	[
		[(ALConnection socket: connection application: application) handleRequests] 
			forkAt: Processor highIOPriority 
			named: 'ALConnection to ', connection remoteAddress printString.
			
	] ifCurtailed: [connection destroy]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/20/2011 23:49'!
shutDown
	state == #running ifTrue:
		[process ifNotNil: [process terminate].
		state := #suspended]! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:35'!
start
	self isRunning ifTrue: [self stop].	
	state := #running.
	process := [self run] 
		forkAt: Processor lowIOPriority 
		named: self printString.
	! !

!ALServer methodsFor: 'running' stamp: 'cwp 5/4/2011 20:30'!
startUp
	state == #suspended ifTrue: [self start]! !

!ALServer methodsFor: 'running' stamp: 'cwp 8/6/2011 16:12'!
stop
	process ifNotNil: [process terminate].
	state := #stopped! !

!ALServer methodsFor: 'printing' stamp: 'cwp 4/13/2011 17:18'!
streamingPrintOn: aWriteStream
	^ aWriteStream
		write: self class name;
		put: $[;
		print: port;
		put: $:;
		write: state;
		put: $];
		conclusion! !

!ALServer methodsFor: 'accessing' stamp: 'cwp 4/13/2011 17:07'!
timeout
	^ 5! !
ALHttpParser initialize!
ALServer initialize!
